==> google-cloud-public-ca-0.2.0.tar.gz: google-cloud-public-ca-0.2.0/google/cloud/security/publicca_v1beta1/services/public_certificate_authority_service/async_client.py <==
# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
import functools
import re
from typing import (
    Dict,
    Mapping,
    MutableMapping,
    MutableSequence,
    Optional,
    Sequence,
    Tuple,
    Type,
    Union,
)

from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.api_core import retry as retries
from google.api_core.client_options import ClientOptions
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore

from google.cloud.security.publicca_v1beta1 import gapic_version as package_version

try:
    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.Retry, object]  # type: ignore

from google.cloud.security.publicca_v1beta1.types import resources, service

from .client import PublicCertificateAuthorityServiceClient
from .transports.base import (
    DEFAULT_CLIENT_INFO,
    PublicCertificateAuthorityServiceTransport,
)
from .transports.grpc_asyncio import (
    PublicCertificateAuthorityServiceGrpcAsyncIOTransport,
)


class PublicCertificateAuthorityServiceAsyncClient:
    """Manages the resources required for ACME `external account
    binding <https://tools.ietf.org/html/rfc8555#section-7.3.4>`__ for
    the public certificate authority service.
    """

    _client: PublicCertificateAuthorityServiceClient

    DEFAULT_ENDPOINT = PublicCertificateAuthorityServiceClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = (
        PublicCertificateAuthorityServiceClient.DEFAULT_MTLS_ENDPOINT
    )

    external_account_key_path = staticmethod(
        PublicCertificateAuthorityServiceClient.external_account_key_path
    )
    parse_external_account_key_path = staticmethod(
        PublicCertificateAuthorityServiceClient.parse_external_account_key_path
    )
    common_billing_account_path = staticmethod(
        PublicCertificateAuthorityServiceClient.common_billing_account_path
    )
    parse_common_billing_account_path = staticmethod(
        PublicCertificateAuthorityServiceClient.parse_common_billing_account_path
    )
    common_folder_path = staticmethod(
        PublicCertificateAuthorityServiceClient.common_folder_path
    )
    parse_common_folder_path = staticmethod(
        PublicCertificateAuthorityServiceClient.parse_common_folder_path
    )
    common_organization_path = staticmethod(
        PublicCertificateAuthorityServiceClient.common_organization_path
    )
    parse_common_organization_path = staticmethod(
        PublicCertificateAuthorityServiceClient.parse_common_organization_path
    )
    common_project_path = staticmethod(
        PublicCertificateAuthorityServiceClient.common_project_path
    )
    parse_common_project_path = staticmethod(
        PublicCertificateAuthorityServiceClient.parse_common_project_path
    )
    common_location_path = staticmethod(
        PublicCertificateAuthorityServiceClient.common_location_path
    )
    parse_common_location_path = staticmethod(
        PublicCertificateAuthorityServiceClient.parse_common_location_path
    )

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            PublicCertificateAuthorityServiceAsyncClient: The constructed client.
        """
        return PublicCertificateAuthorityServiceClient.from_service_account_info.__func__(PublicCertificateAuthorityServiceAsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            PublicCertificateAuthorityServiceAsyncClient: The constructed client.
        """
        return PublicCertificateAuthorityServiceClient.from_service_account_file.__func__(PublicCertificateAuthorityServiceAsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: Optional[ClientOptions] = None
    ):
        """Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variabel is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """
        return PublicCertificateAuthorityServiceClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore

    @property
    def transport(self) -> PublicCertificateAuthorityServiceTransport:
        """Returns the transport used by the client instance.

        Returns:
            PublicCertificateAuthorityServiceTransport: The transport used by the client instance.
        """
        return self._client.transport

    get_transport_class = functools.partial(
        type(PublicCertificateAuthorityServiceClient).get_transport_class,
        type(PublicCertificateAuthorityServiceClient),
    )

    def __init__(
        self,
        *,
        credentials: Optional[ga_credentials.Credentials] = None,
        transport: Union[
            str, PublicCertificateAuthorityServiceTransport
        ] = "grpc_asyncio",
        client_options: Optional[ClientOptions] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the public certificate authority service client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Union[str, ~.PublicCertificateAuthorityServiceTransport]): The
                transport to use. If set to None, a transport is chosen
                automatically.
            client_options (ClientOptions): Custom options for the client. It
                won't take effect if a ``transport`` instance is provided.
                (1) The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client. GOOGLE_API_USE_MTLS_ENDPOINT
                environment variable can also be used to override the endpoint:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto switch to the
                default mTLS endpoint if client certificate is present, this is
                the default value). However, the ``api_endpoint`` property takes
                precedence if provided.
                (2) If GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide client certificate for mutual TLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = PublicCertificateAuthorityServiceClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,
        )

    async def create_external_account_key(
        self,
        request: Optional[Union[service.CreateExternalAccountKeyRequest, dict]] = None,
        *,
        parent: Optional[str] = None,
        external_account_key: Optional[resources.ExternalAccountKey] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> resources.ExternalAccountKey:
        r"""Creates a new
        [ExternalAccountKey][google.cloud.security.publicca.v1beta1.ExternalAccountKey]
        bound to the project.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud.security import publicca_v1beta1

            async def sample_create_external_account_key():
                # Create a client
                client = publicca_v1beta1.PublicCertificateAuthorityServiceAsyncClient()

                # Initialize request argument(s)
                request = publicca_v1beta1.CreateExternalAccountKeyRequest(
                    parent="parent_value",
                )

                # Make the request
                response = await client.create_external_account_key(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.security.publicca_v1beta1.types.CreateExternalAccountKeyRequest, dict]]):
                The request object. Creates a new
                [ExternalAccountKey][google.cloud.security.publicca.v1beta1.ExternalAccountKey]
                in a given project.
            parent (:class:`str`):
                Required. The parent resource where this
                external_account_key will be created. Format:
                projects/[project_id]/locations/[location]. At present
                only the "global" location is supported.

                This corresponds to the ``parent`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            external_account_key (:class:`google.cloud.security.publicca_v1beta1.types.ExternalAccountKey`):
                Required. The external account key to
                create. This field only exists to
                future-proof the API. At present, all
                fields in ExternalAccountKey are output
                only and all values are ignored. For the
                purpose of the
                CreateExternalAccountKeyRequest, set it
                to a default/empty value.

                This corresponds to the ``external_account_key`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.security.publicca_v1beta1.types.ExternalAccountKey:
                A representation of an ExternalAccountKey used for [external account
                   binding](\ https://tools.ietf.org/html/rfc8555#section-7.3.4)
                   within ACME.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([parent, external_account_key])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = service.CreateExternalAccountKeyRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if parent is not None:
            request.parent = parent
        if external_account_key is not None:
            request.external_account_key = external_account_key

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.create_external_account_key,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.transport.close()


DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=package_version.__version__
)


__all__ = ("PublicCertificateAuthorityServiceAsyncClient",)

==> google-cloud-public-ca-0.3.2.tar.gz: google-cloud-public-ca-0.3.2/google/cloud/security/publicca_v1beta1/services/public_certificate_authority_service/async_client.py <==
# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
import functools
import re
from typing import (
    Dict,
    Mapping,
    MutableMapping,
    MutableSequence,
    Optional,
    Sequence,
    Tuple,
    Type,
    Union,
)

from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.api_core import retry as retries
from google.api_core.client_options import ClientOptions
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore

from google.cloud.security.publicca_v1beta1 import gapic_version as package_version

try:
    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.Retry, object]  # type: ignore

from google.cloud.security.publicca_v1beta1.types import resources, service

from .client import PublicCertificateAuthorityServiceClient
from .transports.base import (
    DEFAULT_CLIENT_INFO,
    PublicCertificateAuthorityServiceTransport,
)
from .transports.grpc_asyncio import (
    PublicCertificateAuthorityServiceGrpcAsyncIOTransport,
)


class PublicCertificateAuthorityServiceAsyncClient:
    """Manages the resources required for ACME `external account
    binding <https://tools.ietf.org/html/rfc8555#section-7.3.4>`__ for
    the public certificate authority service.
    """

    _client: PublicCertificateAuthorityServiceClient

    DEFAULT_ENDPOINT = PublicCertificateAuthorityServiceClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = (
        PublicCertificateAuthorityServiceClient.DEFAULT_MTLS_ENDPOINT
    )

    external_account_key_path = staticmethod(
        PublicCertificateAuthorityServiceClient.external_account_key_path
    )
    parse_external_account_key_path = staticmethod(
        PublicCertificateAuthorityServiceClient.parse_external_account_key_path
    )
    common_billing_account_path = staticmethod(
        PublicCertificateAuthorityServiceClient.common_billing_account_path
    )
    parse_common_billing_account_path = staticmethod(
        PublicCertificateAuthorityServiceClient.parse_common_billing_account_path
    )
    common_folder_path = staticmethod(
        PublicCertificateAuthorityServiceClient.common_folder_path
    )
    parse_common_folder_path = staticmethod(
        PublicCertificateAuthorityServiceClient.parse_common_folder_path
    )
    common_organization_path = staticmethod(
        PublicCertificateAuthorityServiceClient.common_organization_path
    )
    parse_common_organization_path = staticmethod(
        PublicCertificateAuthorityServiceClient.parse_common_organization_path
    )
    common_project_path = staticmethod(
        PublicCertificateAuthorityServiceClient.common_project_path
    )
    parse_common_project_path = staticmethod(
        PublicCertificateAuthorityServiceClient.parse_common_project_path
    )
    common_location_path = staticmethod(
        PublicCertificateAuthorityServiceClient.common_location_path
    )
    parse_common_location_path = staticmethod(
        PublicCertificateAuthorityServiceClient.parse_common_location_path
    )

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            PublicCertificateAuthorityServiceAsyncClient: The constructed client.
        """
        return PublicCertificateAuthorityServiceClient.from_service_account_info.__func__(PublicCertificateAuthorityServiceAsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            PublicCertificateAuthorityServiceAsyncClient: The constructed client.
        """
        return PublicCertificateAuthorityServiceClient.from_service_account_file.__func__(PublicCertificateAuthorityServiceAsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: Optional[ClientOptions] = None
    ):
        """Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variable is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """
        return PublicCertificateAuthorityServiceClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore

    @property
    def transport(self) -> PublicCertificateAuthorityServiceTransport:
        """Returns the transport used by the client instance.

        Returns:
            PublicCertificateAuthorityServiceTransport: The transport used by the client instance.
        """
        return self._client.transport

    get_transport_class = functools.partial(
        type(PublicCertificateAuthorityServiceClient).get_transport_class,
        type(PublicCertificateAuthorityServiceClient),
    )

    def __init__(
        self,
        *,
        credentials: Optional[ga_credentials.Credentials] = None,
        transport: Union[
            str, PublicCertificateAuthorityServiceTransport
        ] = "grpc_asyncio",
        client_options: Optional[ClientOptions] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the public certificate authority service client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Union[str, ~.PublicCertificateAuthorityServiceTransport]): The
                transport to use. If set to None, a transport is chosen
                automatically.
            client_options (ClientOptions): Custom options for the client. It
                won't take effect if a ``transport`` instance is provided.
                (1) The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client. GOOGLE_API_USE_MTLS_ENDPOINT
                environment variable can also be used to override the endpoint:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto switch to the
                default mTLS endpoint if client certificate is present, this is
                the default value). However, the ``api_endpoint`` property takes
                precedence if provided.
                (2) If GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide client certificate for mutual TLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = PublicCertificateAuthorityServiceClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,
        )

    async def create_external_account_key(
        self,
        request: Optional[Union[service.CreateExternalAccountKeyRequest, dict]] = None,
        *,
        parent: Optional[str] = None,
        external_account_key: Optional[resources.ExternalAccountKey] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> resources.ExternalAccountKey:
        r"""Creates a new
        [ExternalAccountKey][google.cloud.security.publicca.v1beta1.ExternalAccountKey]
        bound to the project.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud.security import publicca_v1beta1

            async def sample_create_external_account_key():
                # Create a client
                client = publicca_v1beta1.PublicCertificateAuthorityServiceAsyncClient()

                # Initialize request argument(s)
                request = publicca_v1beta1.CreateExternalAccountKeyRequest(
                    parent="parent_value",
                )

                # Make the request
                response = await client.create_external_account_key(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.security.publicca_v1beta1.types.CreateExternalAccountKeyRequest, dict]]):
                The request object. Creates a new
                [ExternalAccountKey][google.cloud.security.publicca.v1beta1.ExternalAccountKey]
                in a given project.
            parent (:class:`str`):
                Required. The parent resource where this
                external_account_key will be created. Format:
                projects/[project_id]/locations/[location]. At present
                only the "global" location is supported.

                This corresponds to the ``parent`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            external_account_key (:class:`google.cloud.security.publicca_v1beta1.types.ExternalAccountKey`):
                Required. The external account key to
                create. This field only exists to
                future-proof the API. At present, all
                fields in ExternalAccountKey are output
                only and all values are ignored. For the
                purpose of the
                CreateExternalAccountKeyRequest, set it
                to a default/empty value.

                This corresponds to the ``external_account_key`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.security.publicca_v1beta1.types.ExternalAccountKey:
                A representation of an ExternalAccountKey used for [external account
                   binding](\ https://tools.ietf.org/html/rfc8555#section-7.3.4)
                   within ACME.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([parent, external_account_key])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = service.CreateExternalAccountKeyRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if parent is not None:
            request.parent = parent
        if external_account_key is not None:
            request.external_account_key = external_account_key

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.create_external_account_key,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.transport.close()


DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=package_version.__version__
)


__all__ = ("PublicCertificateAuthorityServiceAsyncClient",)
